/*
 * packet.java
 * Douglas S. J. De Couto
 * 26 August 1999
 */

/*
 * Copyright (C) 1999 Douglas S. J. De Couto
 * <decouto@lcs.mit.edu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package decouto.garble;

/** Implements packet data structure according to Garmin protocol L000/1.  Used by the link layer and application layer. 
    @author Douglas S. J. De Couto <a href="mailto:decouto@lcs.mit.edu">decouto@lcs.mit.edu</a>
    @version 26 August 1999 0.1
    @see decouto.garble.link_layer 
    @see decouto.garble.application_layer */
class packet {

  /** Maximum number of data bytes in a packet (size is indicated by
      an unsigned byte). */
  static final int m_max_data_sz = 255;

  /** Packet ID. */
  byte id;
  /** Packet size. */
  int sz;
  /** Array of data bytes.  Must be non-null, and contain at least
      m_max_data_sz elements. */
  byte data[];
  
  /** Create a packet with the desired id and size and a valid data
      array, but leave data initialization to the user. 
      @param p_id Packet ID.
      @param p_sz Packet data size in bytes. */
  packet(byte p_id, int p_sz) {
    data = new byte[m_max_data_sz];
    id = p_id;
    sz = p_sz;
  }

  /** Create a packet with an unspecified size and id, but with a
      valid data array, leaving data initialization to the user. */
  packet() {
    data = new byte[m_max_data_sz];
  }

  /** Create a string representation of the packet.
      @return The string, in the following format:
      ``[id_byte sz_byte data_0 ... data_sz-1 calculated_checksum]'' */
  String hex_dump() {
    String s = "[" + Integer.toHexString(id & 0xff) + " " + Integer.toHexString(sz) + " ";
    for (int i = 0; i < sz; i++) {
      s += Integer.toHexString(data[i] & 0xff) + " ";
    }
    s += Integer.toHexString(calc_cksum() & 0xff) + "]";
    return s;
  }

  /** Return the checksum of the packet, calculated as per Garmin
      protocol L000/1.
      @return The checksum byte. */
  byte calc_cksum() {
    int cksum = (id & 0xff) + sz; 
    for (int i = 0; i < sz; i++) { 
      cksum += (data[i] & 0xff);
    }
    cksum = -cksum;
    return (byte) cksum;
  }
	
  /** Packet IDs as defined in Garmin protocol L000/1. */
  static final byte pid_no_packet = 0, // my temporary pid
    // protocol L000
    pid_ack_byte = (byte) 0x06,
    pid_nack_byte = (byte) 0x15,
    pid_protocol_array = (byte) 0xfd,
    pid_product_rqst = (byte) 0xfe,
    pid_product_data = (byte) 0xff,
    
    // protocol L001
    pid_command_data = (byte) 0x0a,
    pid_xfer_cmplt = (byte) 0x0c,
    pid_date_time_data = (byte) 0x0e,
    pid_position_data = (byte) 0x11,
    pid_prx_wpt_data = (byte) 0x13,
    pid_records = (byte) 0x1b,
    pid_rte_hdr = (byte) 0x1d,
    pid_rte_wpt_data = (byte) 0x1e,
    pid_almanac_data = (byte) 0x1f,
    pid_trk_data = (byte) 0x22,
    pid_wpt_data = (byte) 0x23,
    pid_pvt_data = (byte) 0x33,
    
    // protocol L002
    // Note: some packet ids are duplicated from L002, and some
    // ids are the same name but different values from L001.
    pid_almanac_data002 = (byte) 0x04,
    pid_command_data002 = (byte) 0x0c,
    pid_xfer_cmplt002 = (byte) 0x0d,
    pid_date_time_data002 = (byte) 0x14,
    pid_position_data002 = (byte) 0x18,
    pid_records002 = (byte) 0x23,
    pid_rte_hdr002 = (byte) 0x25,
    pid_rte_wpt_data002 = (byte) 0x27,
    pid_wpt_data002 = (byte) 0x2b;

}
